@@ -1,4 +1,4 @@
-/* $Id: c2man.c,v 2.0.1.35 1994/10/06 06:05:26 greyham Exp $
+/* $Id: c2man.c,v 2.0.1.39 1996/03/21 00:38:16 greyham Exp $
  *
  * C Manual page generator.
  * Reads C source code and outputs manual pages.
@@ -85,6 +85,9 @@ static IncludeFile **last_next_include = &first_include;
 ExcludeSection *first_excluded_section;
 static ExcludeSection **last_next_excluded_section = &first_excluded_section;
 
+/* TRUE if c2man should attempt to fixup comment sections */
+boolean fixup_comments = TRUE;
+
 /* do we group related stuff into one file? */
 boolean group_together;
 
@@ -97,6 +100,12 @@ char *group_terse = NULL;
 /* should we always document parameters, even if it's only "Not Documented" */
 boolean always_document_params = TRUE;
 
+/* look for a function def comment at the start of the function body */
+boolean look_at_body_start = FALSE;
+
+/* only look for a function def comment at the start of the function body */
+boolean body_start_only = FALSE;
+
 /* default output info for each object type */
 struct Output_Object_Info output_object[_OBJECT_NUM] =
 {
@@ -117,9 +126,14 @@ struct Output_Object_Info output_object[_OBJECT_NUM] =
 int num_inc_dir = 1;
 const char *inc_dir[MAX_INC_DIR] = { ".\\" };
 #else
+#ifdef AMIGA
+int num_inc_dir = 1;
+const char *inc_dir[MAX_INC_DIR] = { "include:" };
+#else
 int num_inc_dir = 2;
 const char *inc_dir[MAX_INC_DIR] = { "./", "/usr/include/" };
 #endif
+#endif
 
 /* total number of errors encountered */
 int errors;
@@ -132,8 +146,14 @@ boolean inbasefile;	/* are we parsing in that base file? */
 /* is the base file a header file? */
 boolean header_file;
 
+#ifdef AMIGA
+struct Output *output = &autodoc_output;
+const char *default_section = "doc";
+#else
 /* use nroff output by default */
 struct Output *output = &nroff_output;
+const char *default_section = "3";
+#endif
 
 /* should we generate the output file named after the input file? */
 boolean use_input_name = FALSE;
@@ -267,6 +287,9 @@ usage ()
     fputs(" -p\t\tdisable prototype promotion\n", stderr);
     fputs(" -s\t\toutput static declarations\n", stderr);
     fputs(" -v\t\toutput variable declarations\n", stderr);
+    fputs(" -k\t\tdon't attempt to fixup comments\n", stderr);
+    fputs(" -b\t\tlook for descriptions at top of function bodies\n", stderr);
+    fputs(" -B\t\tonly look for descriptions by applying -b\n", stderr);
     fputc('\n', stderr);
     fputs(" -i incfile\n", stderr);
     fputs(" -i \"incfile\"\n", stderr);
@@ -298,13 +321,15 @@ usage ()
 					fputs("file, none or remove\n", stderr);
     fputs(" -n\t\tName output file after input source file\n", stderr);
     fputs(" -L\t\tLazy: Be silent about undocumented parameters\n",
-								    stderr);
-    fputs(" -T n|l|h|t[,options]\tselect typesetting output format: nroff, LaTeX, HTML or TeXinfo\n",
-								    stderr);
+                                                                    stderr);
+
+    fputs(" -T n|l|h|t|a[,options]\tselect typesetting output format: nroff, LaTeX, HTML ,TeXinfo or AutoDoc\n",
+                                                                    stderr);
     nroff_output.print_options();
     latex_output.print_options();
     html_output.print_options();
     texinfo_output.print_options();
+    autodoc_output.print_options();
 
     fputs(" -M name\tset name of the manual in which the page goes\n",
 								    stderr);
@@ -609,7 +634,6 @@ char **argv;
     IncludeFile *includefile;
     ExcludeSection *excludesection;
     char *cpp_opts;
-    const char *default_section = "3";
 #ifdef HAS_LINK
     enum LinkType link_type = LINK_HARD;	/* for -g/G */
 #else
@@ -633,7 +657,7 @@ char **argv;
 #endif /* !VMS  */
 
     /* Scan command line options. */
-    while ((c = getopt(argc, argv, "P:D:F:I:psU:Vvo:eM:H:G:gi:x:S:l:LT:nO:"))
+    while ((c = getopt(argc, argv, "P:D:F:I:psU:Vvo:eM:H:G:gi:x:S:l:LT:nO:kbB"))
 								    != EOF)
     {
 	switch (c) {
@@ -717,6 +741,9 @@ char **argv;
 	case 'v':
 	    variables_out = TRUE;
 	    break;
+	case 'k':
+	    fixup_comments = FALSE;
+	    break;
 	case 'o':
 	    output_dir = optarg;
 	    break;
@@ -770,13 +797,16 @@ char **argv;
 	case 'T':
 	    switch(optarg[0])
 	    {
-	    case 'n':	output = &nroff_output;	break;
+	    case 'n':	output = &nroff_output;	default_section = "3";
+			break;
 	    case 'l':	output = &latex_output;	default_section = "tex";
-            break;
+			break;
 	    case 't':	output = &texinfo_output; default_section = "texi";
 			break;
 	    case 'h':	output = &html_output; default_section = "html";
 			break;
+            case 'a':   output = &autodoc_output; default_section = "doc";
+                        break;
 	    default:	usage();
 	    }
 	    s = strtok(&optarg[1], ",");
@@ -816,6 +846,13 @@ char **argv;
 		output_object[i].subdir = strduplicate(optarg);
 
 	    break;
+	case 'b':
+            look_at_body_start = TRUE;
+            break;
+	case 'B':
+	    body_start_only = TRUE;
+            look_at_body_start = TRUE;
+            break;
 	case '?':
 	default:
 	    usage();
