@@ -19,6 +19,13 @@ void boxcomm_c2(void);
 /***************************************************************************/
 void boxcomm_c3(void);
 
+/*-------------------------------------------------------------------------
+ * C style box comment.
+ *
+ * This function has a dashed C style box comment.
+ *-------------------------------------------------------------------------*/
+void boxcomm_c4(void);
+
 ////////////////////////////////////////////////////////////////////////////
 // C++ style box comment.
 //
