@@ -1,4 +1,4 @@
-/* $Id: manpage.c,v 2.0.1.48 1994/09/16 05:54:36 greyham Exp $
+/* $Id: manpage.c,v 2.0.1.55 1996/03/21 08:24:21 greyham Exp $
  * stuff to do with manual page outputing
  */
 
@@ -160,172 +160,16 @@ void
 output_comment(comment)
 const char *comment;
 {
-    enum { TEXT, PERIOD, CAPITALISE } state = CAPITALISE;
-    boolean new_line = TRUE;
-    boolean dot_command = FALSE;
-    
     if (!comment || !comment[0])
-    {
-	output->text("Not Documented.\n");
-	return;
-    }
-
-    /* correct punctuation a bit as it goes out */
-    for (;*comment;comment++)
-    {
-	int c = *comment;
-
-	if (dot_command)
-	{
-	    if (c == '\n')	dot_command = FALSE;
-	}
-	else if (new_line && c == '.')
-	    dot_command = TRUE;
-	else if (new_line && (c == '-' || c == '*'))
-	{
-	    output->break_line();
-	    state = CAPITALISE;
-	}
-	else if (c == '.')
-	    state = PERIOD;
-	else if (isspace(c) && state == PERIOD)
-	    state = CAPITALISE;
-	else if (isalnum(c) && state == CAPITALISE)
-	{   
-	    if (islower(c))	c = toupper(c);
-	    state = TEXT;
-	}
-           
-	output->character(c);
-	new_line = c == '\n';
-    }
-
-    /* do a full stop if there wasn't one */
-    if (!dot_command && state == TEXT)	output->character('.');
+	output->text("Not Documented.");
+    else if (fixup_comments)
+	output->description(comment);
+    else
+	output->text(comment);
 
     output->character('\n');
 }
 
-/* output the returns section in man page form, followed by a newline */
-void
-output_returns(comment)
-char *comment;
-{
-    enum { TEXT, PERIOD, CAPITALISE } state = CAPITALISE;
-    char lastchar = '\n';
-    boolean tag_list_started = FALSE;
-
-    /* for each line... */
-    while (*comment)
-    {
-	boolean tagged = FALSE;
-
-	/* explicitly reject dot commands */
-	if (*comment && *comment != '.')
-	{
-	    char *c = comment;
-
-	    /* search along until the end of a word */
-	    while (*c && *c != ':' && !isspace(*c))
-		c++;
-
-	    /* skip all spaces or tabs after the first word */
-	    while (*c && *c != '\n')
-	    {
-		if (*c == '\t' || *c == ':')
-		{
-		    tagged = TRUE;
-		    break;
-		}
-		else if (!isspace(*c))
-		    break;
-
-		c++;
-	    }
-	}
-
-	/* is it tagged?; explicitly reject dot commands */
-	if (tagged)
-	{
-	    /* output lingering newline if necessary */
-	    if (lastchar != '\n')
-	    {
-		if (state == TEXT && !ispunct(lastchar))	output->character('.');
-		output->character(lastchar = '\n');
-	    }
-
-	    if (!tag_list_started)
-	    {
-		output->tag_list_start();
-		tag_list_started = TRUE;
-	    }
-
-	    /* output the taggy bit */
-	    output->tag_entry_start();
-	    while (*comment && *comment != ':' && !isspace(*comment))
-		output->character(*comment++);
-	    output->tag_entry_end();
-
-	    /* skip any extra tabs or spaces */
-	    while (*comment == ':' || (isspace(*comment) && *comment != '\n'))
-		comment++;
-
-	    state = CAPITALISE;
-	}
-
-	/* terminate the previous line if necessary */
-	if (lastchar != '\n')	output->character(lastchar = '\n');
-
-	/* dot commands go out unaltered */
-	if (*comment == '.')
-	{
-	    for (;*comment && *comment != '\n'; comment++)
-		output->character(*comment);
-	    output->character('\n');
-	}
-	else
-	{
-	    /* correct punctuation a bit as the line goes out */
-	    for (;*comment && *comment != '\n'; comment++)
-	    {
-		char c = *comment;
-
-		if (c == '.')
-		    state = PERIOD;
-		else if (isspace(c) && state == PERIOD)
-		    state = CAPITALISE;
-		else if (isalnum(c) && state == CAPITALISE)
-		{   
-		    if (islower(c))	c = toupper(c);
-		    state = TEXT;
-		}
-
-		output->character(lastchar = c);
-	    }
-
-	    /* if it ended in punctuation, just output the nl straight away. */
-	    if (ispunct(lastchar))
-	    {
-		if (lastchar == '.')	state = CAPITALISE;
-		output->character(lastchar = '\n');
-	    }
-	}
-
-	if (*comment)	comment++;
-    }
-
-    /* output lingering newline if necessary */
-    if (lastchar != '\n')
-    {
-	if (state == TEXT && !ispunct(lastchar))	output->character('.');
-	output->character('\n');
-    }
-
-    if (tag_list_started)
-	output->tag_list_end();
-
-}
-
 /* output the phrase "a[n] <type name>" */
 void output_conjunction(text)
 char *text;
@@ -523,7 +367,7 @@ char *function;
 	    output->tag_entry_end();
 
 	output_identifier_description(p->declarator->comment, output_comment,
-						&p->decl_spec, &p->declarator);
+						&p->decl_spec, p->declarator);
     }
 
     if (tag_list_started)
@@ -599,8 +443,10 @@ const char *c;
     return c;
 }
 
-/* split the function comment into manual page format */
-void
+/* split the function comment into manual page format.
+ * returns TRUE if the DESCRIPTION field was explicit.
+ */
+boolean
 split_function_comment(comment, identifier_name,
 				    terse, description, returns, extra_sections)
     const char *comment;
@@ -637,12 +483,18 @@ split_function_comment(comment, identifier_name,
 	 */
 	if (lastblank)
 	{
+	    boolean need_colon = FALSE;
+
 	    /* see if we've found the start of a SECTION */
-	    while (isalpha(*c))
+	    while (isalnum(*c) || *c == ' ' || *c == '/')
+	    {
+		if (isspace(*c)) need_colon = TRUE;
 		c++;
+	    }
     
-	    section_heading = *c == '\n' || *c == ':' ||
-					(*c == '\0' && start_line == comment);
+	    section_heading = (!need_colon && *c == '\n') ||
+			(*c == ':' && (!need_colon || *(c+1) == '\n')) ||
+			(!need_colon && *c == '\0' && start_line == comment);
 	}
 	else
 	    section_heading = FALSE;
@@ -781,17 +633,9 @@ split_function_comment(comment, identifier_name,
     /* terminate (or nuke) section list */
     *lastnextsection = NULL;
 
-    /* if there wasn't a RETURNS section, and the DESCRIPTION field was not
-     * explicit, see if we can split one out of the description field.
-     */
-    if (*returns == NULL && !explicit_description)
-    {
-	char *olddesc = *description;
-    	if (split_returns_comment(olddesc, description, returns))
-	    free(olddesc);
-    }
-
     *extra_sections = first_section;
+
+    return explicit_description;
 }
 
 /* see if two parameters are declared identically */
@@ -954,17 +798,52 @@ output_manpage(firstpage, basepage, input_files, title, section)
     ManualPage *page;
     boolean need_returns;
     char *terseout, *terse = NULL;
-    
+    boolean exclude_description = exclude_section("DESCRIPTION");
+
     /* check if there's more than one page in the group */
     boolean grouped = group_together && firstpage->next;
 
     /* split up all the function comments for this page */
     for (page = basepage; page; page = page->next)
     {
-	split_function_comment(page->declarator->comment,
-	    page->declarator->name,
-	    group_together ? (char **)NULL : &terse,
-	    &page->description,&page->returns,&page->first_section);
+	boolean explicit_description =
+	    split_function_comment(page->declarator->comment,
+		page->declarator->name,
+		group_together ? (char **)NULL : &terse,
+		&page->description,&page->returns,&page->first_section);
+
+	/* we may need to look harder if RETURNS wasn't easy to find in the
+	 * function comment.
+	 */
+	if (page->returns == NULL)
+	{
+	    /* if there was a retcomment supplied by the declarator, use it if
+	     * we couldn't split anything from the function comment.
+	     */
+	    if (page->declarator->retcomment)
+	    {
+		page->returns = page->declarator->retcomment;
+
+		/* page->returns now owns the string */
+		page->declarator->retcomment = NULL;
+	    }
+	    else
+		/* if there wasn't a RETURNS section, and the DESCRIPTION field
+		 * was not explicit, see if we can split one out of the
+		 * description field.
+		 */
+		if (!explicit_description)
+		{
+		    char *newdesc;
+		    if (split_returns_comment(page->description, &newdesc,
+							    &page->returns))
+		    {
+			free(page->description);
+			page->description = newdesc;
+		    }
+		}
+	}
+
 	if (!group_together)	break;
     }
 
@@ -972,7 +851,7 @@ output_manpage(firstpage, basepage, input_files, title, section)
     terseout = group_terse ? group_terse : (terse ? terse : "Not Described");
 
     output->header(basepage, input_files, grouped,
-		    title ? title : basepage->declarator->name, section);
+		title ? title : basepage->declarator->name, terseout, section);
     
     output->name(NULL);
     /* output the names of all the stuff documented on this page */
@@ -989,60 +868,64 @@ output_manpage(firstpage, basepage, input_files, title, section)
     output->text(terseout);
     output->character('\n');
     
-    output->section("SYNOPSIS");
-
-    output->code_start();
-    
-    /* list the include files the user asked us to */
-    output_includes();
-
-    /* if it's a header file, say to #include it */
-    if (header_file)
+    if (!exclude_section("SYNOPSIS"))
     {
-	output->text("#include <");
-	if (header_prefix)
+	output->section("SYNOPSIS");
+
+	output->code_start();
+    
+	/* list the include files the user asked us to */
+	output_includes();
+
+	/* if it's a header file, say to #include it */
+	if (header_file)
 	{
-	    output->text(header_prefix);
-	    output->character('/');
+	    output->text("#include <");
+	    if (header_prefix)
+	    {
+		output->text(header_prefix);
+		output->character('/');
+	    }
+	    output->text(basefile);
+	    output->text(">\n");
 	}
-	output->text(basefile);
-	output->text(">\n");
-    }
 
-    /* can't just use .PP; that may reset our font */
-    if (first_include || header_file)	output->blank_line();
+	/* can't just use .PP; that may reset our font */
+	if (first_include || header_file)	output->blank_line();
 
-    for (page = basepage; page; page = page->next)
-    {
-	output_format_string(decl_spec_prefix);
+	for (page = basepage; page; page = page->next)
+	{
+	    output_format_string(decl_spec_prefix);
 
-	/* make sure variables are prefixed extern */
-	if (!(page->decl_spec->flags & DS_STATIC) &&
-	    !is_function_declarator(page->declarator) &&
-	    !strstr(page->decl_spec->text, "extern"))
-	    output->text("extern ");
+	    /* make sure variables are prefixed extern */
+	    if (!(page->decl_spec->flags & DS_STATIC) &&
+		!is_function_declarator(page->declarator) &&
+		!strstr(page->decl_spec->text, "extern"))
+		output->text("extern ");
 
-	output_decl_spec(page->decl_spec);
-	output_format_string(declarator_prefix);
+	    output_decl_spec(page->decl_spec);
+	    output_format_string(declarator_prefix);
 
-	/* format it nicely if there's more than one parameter */
-	output_declarator(page->declarator, 
-	    page->declarator->head->params.first !=
-	    page->declarator->head->params.last);
+	    /* format it nicely if there's more than one parameter */
+	    output_declarator(page->declarator, 
+		page->declarator->head->params.first !=
+		page->declarator->head->params.last);
 
-	output->text(";\n");
+	    output->text(";\n");
     
-	if (!grouped)	break;
-	if (page->next)	output->blank_line();
-    }
+	    if (!grouped)	break;
+	    if (page->next)	output->blank_line();
+	}
 
-    output->code_end();
+	output->code_end();
+    }
 
     /* only output paramaters if there actually are some,
      * not including merely (void)
      */
-    if ((grouped && mark_duplicate_parameters(basepage)) ||
-        (!grouped && has_documented_parameters(basepage->declarator)))
+    if (!exclude_section("PARAMETERS") &&
+	    ((grouped && mark_duplicate_parameters(basepage)) ||
+	     (!grouped && has_documented_parameters(basepage->declarator))))
     {
 	output->section("PARAMETERS");
 
@@ -1055,7 +938,8 @@ output_manpage(firstpage, basepage, input_files, title, section)
 	}
     }
 
-    output->section("DESCRIPTION");
+    if (!exclude_description)
+	output->section("DESCRIPTION");
 
     if (grouped)
     {
@@ -1064,18 +948,21 @@ output_manpage(firstpage, basepage, input_files, title, section)
 	{
 	    if (needs_returns_section(page))	need_returns = TRUE;
 
-	    /* enum variables are documented in DESCRIPTION */
-	    if (auto_documented(page) &&
+	    if (!exclude_description)
+	    {
+		/* enum variables are documented in DESCRIPTION */
+		if (auto_documented(page) &&
 				    !is_function_declarator(page->declarator))
-	    {
-		output->sub_section(page->declarator->name);
-		output_identifier_description(page->description,
-		    output_comment, page->decl_spec, page->declarator);
-	    }
-	    else if (page->description)
-	    {
-		output->sub_section(page->declarator->name);
-		output_comment(page->description);
+		{
+		    output->sub_section(page->declarator->name);
+		    output_identifier_description(page->description,
+			output_comment, page->decl_spec, page->declarator);
+		}
+		else if (page->description)
+		{
+		    output->sub_section(page->declarator->name);
+		    output_comment(page->description);
+		}
 	    }
 
 	    safe_free(page->description);
@@ -1083,25 +970,28 @@ output_manpage(firstpage, basepage, input_files, title, section)
     }
     else
     {
-	const char *descr = basepage->description ? basepage->description
-						   : terseout;
-
 	need_returns = needs_returns_section(basepage);
 
-	if (auto_documented(page) && !is_function_declarator(page->declarator))
-	    output_identifier_description(descr, output_comment,
+	if (!exclude_description)
+	{
+	    const char *descr = basepage->description
+					? basepage->description : terseout;
+
+	    if (auto_documented(page) &&
+				    !is_function_declarator(page->declarator))
+		output_identifier_description(descr, output_comment,
 					    page->decl_spec, page->declarator);
-	else
-	    output_comment(descr);
+	    else
+		output_comment(descr);
+	}
 
 	safe_free(basepage->description);
-
     }
 
     /* terse can now never be a static string */
     safe_free(terse);
 
-    if (need_returns)
+    if (need_returns && !exclude_section("RETURNS"))
     {
 	output->section("RETURNS");
 
@@ -1111,7 +1001,7 @@ output_manpage(firstpage, basepage, input_files, title, section)
 	    {
 		if (grouped) output->sub_section(page->declarator->name);
 
-		output_identifier_description(page->returns, output_returns,
+		output_identifier_description(page->returns, output->returns,
 					    page->decl_spec, page->declarator);
 		safe_free(page->returns);
 	    }
